// Vertex Shader â€“ file "base.vert"

#version 140 //SL version 2 not 4

in  vec3 in_Position;
in  vec3 in_Color;
in  vec3 in_Normal;
in 	vec3 in_FlatNormal;
in  vec3 in_Tangent;
in	vec2 in_TexCoord;

out vec2 ex_fragTexCoord;
out vec3 fragNormal;
out vec3 fragEye;
out vec3 light;
out vec4 sm_tex_coord;
out vec3 lv;
out vec3 eyeVec;
out vec4 old_Position;

	
uniform mat4 mvMat;
uniform mat4 invmvMat;
uniform mat4 V_cam;
uniform mat4 M;
uniform mat4 V_shadow;
uniform mat4 P_shadow;
uniform mat4 projMat;
uniform mat4 texture_matrix;
uniform vec3 lightDirection;
uniform float timer;

uniform int mode;

vec3 getFragNormal(){
	// for mode 0, 1 and 2 return the flat normals
	return (mode > 2) ? (mvMat * vec4(in_Normal, 0.0f)).xyz: mat3(mvMat) * in_FlatNormal;
}
vec4 getSMTexCoord(){
	//shadow map stuff here
	mat4 bias = mat4( 0.5, 0.0, 0.0, 0.0,
					  0.0, 0.5, 0.0, 0.0,
					  0.0, 0.0, 0.5, 0.0,
					  0.5, 0.5, 0.5, 1.0 );
	return bias * P_shadow * V_shadow * M * vec4(in_Position,1);
}
mat3 getNTBmatrix(){
	// Building the normal matrix 
	vec3 n = normalize (mat3(mvMat) * in_Normal);
	vec3 t = normalize (mat3(mvMat) * in_Tangent);
	vec3 b = cross (n, t);
	return mat3(n,t,b);
}
void main(void)
{

if(mode == -2) {
gl_Position = projMat *  mvMat * vec4(in_Position,1);
}
else{
	old_Position = vec4(in_Position,1);
	light = (mvMat * vec4(lightDirection,1)).xyz - (mvMat * vec4(in_Position,1)).xyz; 
	//light = (mvMat * vec4(lightDirection,0)).xyz;
	
	
	fragNormal = getFragNormal();
	
	fragEye = (mvMat * vec4(in_Position, 1)).xyz;
	fragEye = -normalize(fragEye);
	
	ex_fragTexCoord.x = in_TexCoord.x;
	ex_fragTexCoord.y = -1 * in_TexCoord.y;


	//position calculation
	gl_Position = projMat *  mvMat * vec4(in_Position,1);
	vec4 pos = vec4(in_Position,1);
	pos.y += (sin(in_Position.x + timer)+cos(in_Position.z + timer))/10;
	if(mode == 6) gl_Position = projMat *  mvMat * pos;
	
	if(mode == 7 ){
		pos = M * vec4(in_Position, 1);
		float waterlvl = 25; //27.6
		pos.y = pos.y * -1 - waterlvl;
		gl_Position = projMat *  V_cam * pos; //flip y coordinate for mirror
	}
	
	
	
	sm_tex_coord = getSMTexCoord();
	//gl_Position = P_shadow * V_shadow * M * vec4(in_Position,1);
	
	// bump mapping
	if (mode == 5 || mode == 6){
	/// ----- bump mapping
		//mat3 ntbMatrix = getNTBmatrix();
		mat4 invtransmv = mvMat;
		invtransmv[0][3] = -mvMat[0][3];
		invtransmv[1][3] = -mvMat[1][3];
		invtransmv[2][3] = -mvMat[2][3];
		vec3 n = normalize (mat3(invtransmv) * in_Normal);
	    vec3 t = normalize (mat3(invtransmv) * in_Tangent);
	    vec3 b = cross (n, t);
	    
	    lv.x = dot(light, t);
	    lv.y = dot(light, b);
	    lv.z = dot(light, n);
	    lv = normalize(lv);
	    
	    eyeVec.x = dot(fragEye, t);
	    eyeVec.y = dot(fragEye, b);
	    eyeVec.z = dot(fragEye, n);
	    eyeVec = normalize(eyeVec);
	    
	    
	    
		//vec3 lightDir = normalize(light.xyz - (mvMat * vec4(in_Position, 1)).xyz);
		
		//lv = normalize ( ntbMatrix * light );
		//eyeVec = normalize ( ntbMatrix * (mvMat * vec4(in_Position, 1)).xyz);
		
	}
}
}